@@ -141,7 +141,7 @@ class TestGeneratorClassic:
     def test_binomial_tree(self):
         graphs = (None, nx.Graph, nx.DiGraph, nx.MultiGraph, nx.MultiDiGraph)
         for create_using in graphs:
-            for n in range(0, 4):
+            for n in range(4):
                 b = nx.binomial_tree(n, create_using)
                 assert nx.number_of_nodes(b) == 2**n
                 assert nx.number_of_edges(b) == (2**n - 1)
@@ -473,6 +473,67 @@ class TestGeneratorClassic:
         assert len(G) == 4
         assert G.size() == 3
 
+    def test_tadpole_graph_right_sizes(self):
+        # number of nodes = m1 + m2
+        # number of edges = m1 + m2 - (m1 == 2)
+        for m1, m2 in [(3, 0), (3, 5), (4, 10), (3, 20)]:
+            G = nx.tadpole_graph(m1, m2)
+            assert nx.number_of_nodes(G) == m1 + m2
+            assert nx.number_of_edges(G) == m1 + m2 - (m1 == 2)
+        for first, second in [("ab", ""), ("ab", "c"), ("abc", "defg")]:
+            m1, m2 = len(first), len(second)
+            print(first, second)
+            G = nx.tadpole_graph(first, second)
+            print(G.edges())
+            assert nx.number_of_nodes(G) == m1 + m2
+            assert nx.number_of_edges(G) == m1 + m2 - (m1 == 2)
+
+    def test_tadpole_graph_exceptions(self):
+        # Raise NetworkXError if m<2
+        pytest.raises(nx.NetworkXError, nx.tadpole_graph, -1, 3)
+        pytest.raises(nx.NetworkXError, nx.tadpole_graph, 0, 3)
+        pytest.raises(nx.NetworkXError, nx.tadpole_graph, 1, 3)
+
+        # Raise NetworkXError if n<0
+        pytest.raises(nx.NetworkXError, nx.tadpole_graph, 5, -2)
+
+        # Raise NetworkXError for digraphs
+        with pytest.raises(nx.NetworkXError):
+            nx.tadpole_graph(2, 20, create_using=nx.DiGraph)
+        with pytest.raises(nx.NetworkXError):
+            nx.tadpole_graph(2, 20, create_using=nx.MultiDiGraph)
+
+    def test_tadpole_graph_same_as_path_when_m1_is_2_or_0(self):
+        # tadpole_graph(2,m) = path_graph(m+2)
+        for m1, m2 in [(2, 0), (2, 5), (2, 10), ("ab", 20)]:
+            G = nx.tadpole_graph(m1, m2)
+            assert is_isomorphic(G, nx.path_graph(m2 + 2))
+
+    def test_tadpole_graph_same_as_cycle_when_m2_is_0(self):
+        # tadpole_graph(m,0) = cycle_(m)
+        for m1, m2 in [(4, 0), (7, 0)]:
+            G = nx.tadpole_graph(m1, m2)
+            assert is_isomorphic(G, nx.cycle_graph(m1))
+
+    def test_tadpole_graph_for_multigraph(self):
+        G = nx.tadpole_graph(5, 20)
+        MG = nx.tadpole_graph(5, 20, create_using=nx.MultiGraph)
+        assert edges_equal(MG.edges(), G.edges())
+
+    def test_tadpole_graph_mixing_input_types(self):
+        cases = [(4, "abc"), ("abcd", 3), ([1, 2, 3, 4], "abc"), ("abcd", [1, 2, 3])]
+        for m1, m2 in cases:
+            G = nx.tadpole_graph(m1, m2)
+            assert len(G) == 7
+            assert G.size() == 7
+
+    def test_tadpole_graph_not_int_integer_inputs(self):
+        # test non-int integers
+        np = pytest.importorskip("numpy")
+        G = nx.tadpole_graph(np.int32(4), np.int64(3))
+        assert len(G) == 7
+        assert G.size() == 7
+
     def test_trivial_graph(self):
         assert nx.number_of_nodes(nx.trivial_graph()) == 1
 
@@ -557,3 +618,5 @@ class TestGeneratorClassic:
             for u, v in itertools.product(block1, block2):
                 assert v in G[u]
                 assert G.nodes[u] != G.nodes[v]
+        with pytest.raises(nx.NetworkXError, match="Negative number of nodes"):
+            nx.complete_multipartite_graph(2, -3, 4)
